@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: access32.cc,v 1.27 2010/03/14 15:51:26 sshwarts Exp $
+// $Id: access32.cc,v 1.31 2011/01/04 16:17:20 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2008-2009 Stanislav Shwartsman
+//   Copyright (c) 2008-2010 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -48,11 +48,11 @@ accessOK:
         if (! (tlbEntry->accessBits & (0x2 | USER_PL))) {
           bx_hostpageaddr_t hostPageAddr = tlbEntry->hostPageAddr;
           Bit32u pageOffset = PAGE_OFFSET(laddr);
-          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, tlbEntry->ppf | pageOffset, 1, BX_WRITE);
-          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr,
-              tlbEntry->ppf | pageOffset, 1, CPL, BX_WRITE, (Bit8u*) &data);
+          bx_phy_address pAddr = tlbEntry->ppf | pageOffset;
+          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, pAddr, 1, BX_WRITE);
+          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr, pAddr, 1, CPL, BX_WRITE, (Bit8u*) &data);
           Bit8u *hostAddr = (Bit8u*) (hostPageAddr | pageOffset);
-          pageWriteStampTable.decWriteStamp(tlbEntry->ppf);
+          pageWriteStampTable.decWriteStamp(pAddr, 1);
           *hostAddr = data;
           return;
         }
@@ -97,11 +97,11 @@ accessOK:
         if (! (tlbEntry->accessBits & (0x2 | USER_PL))) {
           bx_hostpageaddr_t hostPageAddr = tlbEntry->hostPageAddr;
           Bit32u pageOffset = PAGE_OFFSET(laddr);
-          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, tlbEntry->ppf | pageOffset, 2, BX_WRITE);
-          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr,
-              tlbEntry->ppf | pageOffset, 2, CPL, BX_WRITE, (Bit8u*) &data);
+          bx_phy_address pAddr = tlbEntry->ppf | pageOffset;
+          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, pAddr, 2, BX_WRITE);
+          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr, pAddr, 2, CPL, BX_WRITE, (Bit8u*) &data);
           Bit16u *hostAddr = (Bit16u*) (hostPageAddr | pageOffset);
-          pageWriteStampTable.decWriteStamp(tlbEntry->ppf);
+          pageWriteStampTable.decWriteStamp(pAddr, 2);
           WriteHostWordToLittleEndian(hostAddr, data);
           return;
         }
@@ -156,11 +156,11 @@ accessOK:
         if (! (tlbEntry->accessBits & (0x2 | USER_PL))) {
           bx_hostpageaddr_t hostPageAddr = tlbEntry->hostPageAddr;
           Bit32u pageOffset = PAGE_OFFSET(laddr);
-          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, tlbEntry->ppf | pageOffset, 4, BX_WRITE);
-          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr,
-              tlbEntry->ppf | pageOffset, 4, CPL, BX_WRITE, (Bit8u*) &data);
+          bx_phy_address pAddr = tlbEntry->ppf | pageOffset;
+          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, pAddr, 4, BX_WRITE);
+          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr, pAddr, 4, CPL, BX_WRITE, (Bit8u*) &data);
           Bit32u *hostAddr = (Bit32u*) (hostPageAddr | pageOffset);
-          pageWriteStampTable.decWriteStamp(tlbEntry->ppf);
+          pageWriteStampTable.decWriteStamp(pAddr, 4);
           WriteHostDWordToLittleEndian(hostAddr, data);
           return;
         }
@@ -215,11 +215,11 @@ accessOK:
         if (! (tlbEntry->accessBits & (0x2 | USER_PL))) {
           bx_hostpageaddr_t hostPageAddr = tlbEntry->hostPageAddr;
           Bit32u pageOffset = PAGE_OFFSET(laddr);
-          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, tlbEntry->ppf | pageOffset, 8, BX_WRITE);
-          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr,
-              tlbEntry->ppf | pageOffset, 8, CPL, BX_WRITE, (Bit8u*) &data);
+          bx_phy_address pAddr = tlbEntry->ppf | pageOffset;
+          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, pAddr, 8, BX_WRITE);
+          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr, pAddr, 8, CPL, BX_WRITE, (Bit8u*) &data);
           Bit64u *hostAddr = (Bit64u*) (hostPageAddr | pageOffset);
-          pageWriteStampTable.decWriteStamp(tlbEntry->ppf);
+          pageWriteStampTable.decWriteStamp(pAddr, 8);
           WriteHostQWordToLittleEndian(hostAddr, data);
           return;
         }
@@ -272,11 +272,11 @@ accessOK:
         if (! (tlbEntry->accessBits & (0x2 | USER_PL))) {
           bx_hostpageaddr_t hostPageAddr = tlbEntry->hostPageAddr;
           Bit32u pageOffset = PAGE_OFFSET(laddr);
-          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, tlbEntry->ppf | pageOffset, 16, BX_WRITE);
-          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr,
-            tlbEntry->ppf | pageOffset, 16, CPL, BX_WRITE, (Bit8u*) data);
+          bx_phy_address pAddr = tlbEntry->ppf | pageOffset;
+          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, pAddr, 16, BX_WRITE);
+          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr, pAddr, 16, CPL, BX_WRITE, (Bit8u*) data);
           Bit64u *hostAddr = (Bit64u*) (hostPageAddr | pageOffset);
-          pageWriteStampTable.decWriteStamp(tlbEntry->ppf);
+          pageWriteStampTable.decWriteStamp(pAddr, 16);
           WriteHostQWordToLittleEndian(hostAddr,   data->xmm64u(0));
           WriteHostQWordToLittleEndian(hostAddr+1, data->xmm64u(1));
           return;
@@ -300,18 +300,24 @@ accessOK:
   void BX_CPP_AttrRegparmN(3)
 BX_CPU_C::write_virtual_dqword_aligned_32(unsigned s, Bit32u offset, const BxPackedXmmRegister *data)
 {
-  Bit32u laddr;
   bx_segment_reg_t *seg = &BX_CPU_THIS_PTR sregs[s];
   BX_INSTR_MEM_DATA_ACCESS(BX_CPU_ID, s, offset, 16, BX_WRITE);
 
   BX_ASSERT(BX_CPU_THIS_PTR cpu_mode != BX_MODE_LONG_64);
 
+  Bit32u laddr = BX_CPU_THIS_PTR get_laddr32(s, offset);
+  // must check alignment here because #GP on misaligned access is higher
+  // priority than other segment related faults
+  if (laddr & 15) {
+    BX_ERROR(("write_virtual_dqword_aligned_32(): #GP misaligned access"));
+    exception(BX_GP_EXCEPTION, 0);
+  }
+
   if (seg->cache.valid & SegAccessWOK) {
     if (offset <= (seg->cache.u.segment.limit_scaled-15)) {
 accessOK:
-      laddr = BX_CPU_THIS_PTR get_laddr32(s, offset);
       unsigned tlbIndex = BX_TLB_INDEX_OF(laddr, 0);
-      Bit32u lpf = AlignedAccessLPFOf(laddr, 15);
+      Bit32u lpf = LPFOf(laddr);
       bx_TLB_entry *tlbEntry = &BX_CPU_THIS_PTR TLB.entry[tlbIndex];
       if (tlbEntry->lpf == lpf) {
         // See if the TLB entry privilege level allows us write access
@@ -319,20 +325,16 @@ accessOK:
         if (! (tlbEntry->accessBits & (0x2 | USER_PL))) {
           bx_hostpageaddr_t hostPageAddr = tlbEntry->hostPageAddr;
           Bit32u pageOffset = PAGE_OFFSET(laddr);
-          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, tlbEntry->ppf | pageOffset, 16, BX_WRITE);
-          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr,
-            tlbEntry->ppf | pageOffset, 16, CPL, BX_WRITE, (Bit8u*) data);
+          bx_phy_address pAddr = tlbEntry->ppf | pageOffset;
+          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, pAddr, 16, BX_WRITE);
+          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr, pAddr, 16, CPL, BX_WRITE, (Bit8u*) data);
           Bit64u *hostAddr = (Bit64u*) (hostPageAddr | pageOffset);
-          pageWriteStampTable.decWriteStamp(tlbEntry->ppf);
+          pageWriteStampTable.decWriteStamp(pAddr, 16);
           WriteHostQWordToLittleEndian(hostAddr,   data->xmm64u(0));
           WriteHostQWordToLittleEndian(hostAddr+1, data->xmm64u(1));
           return;
         }
       }
-      if (laddr & 15) {
-        BX_ERROR(("write_virtual_dqword_aligned_32(): #GP misaligned access"));
-        exception(BX_GP_EXCEPTION, 0);
-      }
       access_write_linear(laddr, 16, CPL, (void *) data);
       return;
     }
@@ -621,18 +623,24 @@ accessOK:
   void BX_CPP_AttrRegparmN(3)
 BX_CPU_C::read_virtual_dqword_aligned_32(unsigned s, Bit32u offset, BxPackedXmmRegister *data)
 {
-  Bit32u laddr;
   bx_segment_reg_t *seg = &BX_CPU_THIS_PTR sregs[s];
   BX_INSTR_MEM_DATA_ACCESS(BX_CPU_ID, s, offset, 16, BX_READ);
 
   BX_ASSERT(BX_CPU_THIS_PTR cpu_mode != BX_MODE_LONG_64);
 
+  Bit32u laddr = BX_CPU_THIS_PTR get_laddr32(s, offset);
+  // must check alignment here because #GP on misaligned access is higher
+  // priority than other segment related faults
+  if (laddr & 15) {
+    BX_ERROR(("read_virtual_dqword_aligned_32(): #GP misaligned access"));
+    exception(BX_GP_EXCEPTION, 0);
+  }
+
   if (seg->cache.valid & SegAccessROK) {
     if (offset <= (seg->cache.u.segment.limit_scaled-15)) {
 accessOK:
-      laddr = BX_CPU_THIS_PTR get_laddr32(s, offset);
       unsigned tlbIndex = BX_TLB_INDEX_OF(laddr, 0);
-      Bit32u lpf = AlignedAccessLPFOf(laddr, 15);
+      Bit32u lpf = LPFOf(laddr);
       bx_TLB_entry *tlbEntry = &BX_CPU_THIS_PTR TLB.entry[tlbIndex];
       if (tlbEntry->lpf == lpf) {
         // See if the TLB entry privilege level allows us read access
@@ -649,10 +657,6 @@ accessOK:
           return;
         }
       }
-      if (laddr & 15) {
-        BX_ERROR(("read_virtual_dqword_aligned_32(): #GP misaligned access"));
-        exception(BX_GP_EXCEPTION, 0);
-      }
       access_read_linear(laddr, 16, CPL, BX_READ, (void *) data);
       return;
     }
@@ -697,13 +701,13 @@ accessOK:
         if (! (tlbEntry->accessBits & (0x2 | USER_PL))) {
           bx_hostpageaddr_t hostPageAddr = tlbEntry->hostPageAddr;
           Bit32u pageOffset = PAGE_OFFSET(laddr);
+          bx_phy_address pAddr = tlbEntry->ppf | pageOffset;
           Bit8u *hostAddr = (Bit8u*) (hostPageAddr | pageOffset);
-          pageWriteStampTable.decWriteStamp(tlbEntry->ppf);
+          pageWriteStampTable.decWriteStamp(pAddr, 1);
           data = *hostAddr;
           BX_CPU_THIS_PTR address_xlation.pages = (bx_ptr_equiv_t) hostAddr;
-          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, tlbEntry->ppf | pageOffset, 1, BX_RW);
-          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr,
-              tlbEntry->ppf | pageOffset, 1, CPL, BX_READ, (Bit8u*) &data);
+          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, pAddr, 1, BX_RW);
+          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr, pAddr, 1, CPL, BX_READ, (Bit8u*) &data);
           return data;
         }
       }
@@ -748,13 +752,13 @@ accessOK:
         if (! (tlbEntry->accessBits & (0x2 | USER_PL))) {
           bx_hostpageaddr_t hostPageAddr = tlbEntry->hostPageAddr;
           Bit32u pageOffset = PAGE_OFFSET(laddr);
+          bx_phy_address pAddr = tlbEntry->ppf | pageOffset;
           Bit16u *hostAddr = (Bit16u*) (hostPageAddr | pageOffset);
-          pageWriteStampTable.decWriteStamp(tlbEntry->ppf);
+          pageWriteStampTable.decWriteStamp(pAddr, 2);
           ReadHostWordFromLittleEndian(hostAddr, data);
           BX_CPU_THIS_PTR address_xlation.pages = (bx_ptr_equiv_t) hostAddr;
-          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, tlbEntry->ppf | pageOffset, 2, BX_RW);
-          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr,
-              tlbEntry->ppf | pageOffset, 2, CPL, BX_READ, (Bit8u*) &data);
+          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, pAddr, 2, BX_RW);
+          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr, pAddr, 2, CPL, BX_READ, (Bit8u*) &data);
           return data;
         }
       }
@@ -809,13 +813,13 @@ accessOK:
         if (! (tlbEntry->accessBits & (0x2 | USER_PL))) {
           bx_hostpageaddr_t hostPageAddr = tlbEntry->hostPageAddr;
           Bit32u pageOffset = PAGE_OFFSET(laddr);
+          bx_phy_address pAddr = tlbEntry->ppf | pageOffset;
           Bit32u *hostAddr = (Bit32u*) (hostPageAddr | pageOffset);
-          pageWriteStampTable.decWriteStamp(tlbEntry->ppf);
+          pageWriteStampTable.decWriteStamp(pAddr, 4);
           ReadHostDWordFromLittleEndian(hostAddr, data);
           BX_CPU_THIS_PTR address_xlation.pages = (bx_ptr_equiv_t) hostAddr;
-          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, tlbEntry->ppf | pageOffset, 4, BX_RW);
-          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr,
-              tlbEntry->ppf | pageOffset, 4, CPL, BX_READ, (Bit8u*) &data);
+          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, pAddr, 4, BX_RW);
+          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr, pAddr, 4, CPL, BX_READ, (Bit8u*) &data);
           return data;
         }
       }
@@ -870,13 +874,13 @@ accessOK:
         if (! (tlbEntry->accessBits & (0x2 | USER_PL))) {
           bx_hostpageaddr_t hostPageAddr = tlbEntry->hostPageAddr;
           Bit32u pageOffset = PAGE_OFFSET(laddr);
+          bx_phy_address pAddr = tlbEntry->ppf | pageOffset;
           Bit64u *hostAddr = (Bit64u*) (hostPageAddr | pageOffset);
-          pageWriteStampTable.decWriteStamp(tlbEntry->ppf);
+          pageWriteStampTable.decWriteStamp(pAddr, 8);
           ReadHostQWordFromLittleEndian(hostAddr, data);
           BX_CPU_THIS_PTR address_xlation.pages = (bx_ptr_equiv_t) hostAddr;
-          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, tlbEntry->ppf | pageOffset, 8, BX_RW);
-          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr,
-              tlbEntry->ppf | pageOffset, 8, CPL, BX_READ, (Bit8u*) &data);
+          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, pAddr, 8, BX_RW);
+          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr, pAddr, 8, CPL, BX_READ, (Bit8u*) &data);
           return data;
         }
       }
@@ -1075,11 +1079,11 @@ accessOK:
         if (! (tlbEntry->accessBits & (0x2 | user))) {
           bx_hostpageaddr_t hostPageAddr = tlbEntry->hostPageAddr;
           Bit32u pageOffset = PAGE_OFFSET(laddr);
-          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, tlbEntry->ppf | pageOffset, 2, BX_WRITE);
-          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr,
-            tlbEntry->ppf | pageOffset, 2, curr_pl, BX_WRITE, (Bit8u*) &data);
+          bx_phy_address pAddr = tlbEntry->ppf | pageOffset;
+          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, pAddr, 2, BX_WRITE);
+          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr, pAddr, 2, curr_pl, BX_WRITE, (Bit8u*) &data);
           Bit16u *hostAddr = (Bit16u*) (hostPageAddr | pageOffset);
-          pageWriteStampTable.decWriteStamp(tlbEntry->ppf);
+          pageWriteStampTable.decWriteStamp(pAddr, 2);
           WriteHostWordToLittleEndian(hostAddr, data);
           return;
         }
@@ -1134,11 +1138,11 @@ accessOK:
         if (! (tlbEntry->accessBits & (0x2 | user))) {
           bx_hostpageaddr_t hostPageAddr = tlbEntry->hostPageAddr;
           Bit32u pageOffset = PAGE_OFFSET(laddr);
-          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, tlbEntry->ppf | pageOffset, 4, BX_WRITE);
-          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr,
-            tlbEntry->ppf | pageOffset, 4, curr_pl, BX_WRITE, (Bit8u*) &data);
+          bx_phy_address pAddr = tlbEntry->ppf | pageOffset;
+          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, pAddr, 4, BX_WRITE);
+          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr, pAddr, 4, curr_pl, BX_WRITE, (Bit8u*) &data);
           Bit32u *hostAddr = (Bit32u*) (hostPageAddr | pageOffset);
-          pageWriteStampTable.decWriteStamp(tlbEntry->ppf);
+          pageWriteStampTable.decWriteStamp(pAddr, 4);
           WriteHostDWordToLittleEndian(hostAddr, data);
           return;
         }
@@ -1193,11 +1197,11 @@ accessOK:
         if (! (tlbEntry->accessBits & (0x2 | user))) {
           bx_hostpageaddr_t hostPageAddr = tlbEntry->hostPageAddr;
           Bit32u pageOffset = PAGE_OFFSET(laddr);
-          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, tlbEntry->ppf | pageOffset, 8, BX_WRITE);
-          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr,
-              tlbEntry->ppf | pageOffset, 8, curr_pl, BX_WRITE, (Bit8u*) &data);
+          bx_phy_address pAddr = tlbEntry->ppf | pageOffset;
+          BX_INSTR_LIN_ACCESS(BX_CPU_ID, laddr, pAddr, 8, BX_WRITE);
+          BX_DBG_LIN_MEMORY_ACCESS(BX_CPU_ID, laddr, pAddr, 8, curr_pl, BX_WRITE, (Bit8u*) &data);
           Bit64u *hostAddr = (Bit64u*) (hostPageAddr | pageOffset);
-          pageWriteStampTable.decWriteStamp(tlbEntry->ppf);
+          pageWriteStampTable.decWriteStamp(pAddr, 8);
           WriteHostQWordToLittleEndian(hostAddr, data);
           return;
         }
