@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: apic.h,v 1.49 2009/10/14 20:45:29 sshwarts Exp $
+// $Id: apic.h,v 1.55 2010/04/08 15:50:39 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (c) 2002-2009 Zwane Mwaikambo, Stanislav Shwartsman
@@ -28,12 +28,6 @@
 #define APIC_LEVEL_TRIGGERED	1
 #define APIC_EDGE_TRIGGERED	0
 
-#ifdef BX_IMPLEMENT_XAPIC
-#  define BX_LAPIC_VERSION_ID 0x00050014  // P4 has 6 LVT entries
-#else
-#  define BX_LAPIC_VERSION_ID 0x00040010  // P6 has 4 LVT entries
-#endif
-
 #define BX_LAPIC_BASE_ADDR  0xfee00000  // default Local APIC address
 #define BX_NUM_LOCAL_APICS  BX_SMP_PROCESSORS
 #define BX_LAPIC_MAX_INTS   256
@@ -43,11 +37,16 @@
 #define BX_APIC_XAPIC_MODE        2
 #define BX_APIC_X2APIC_MODE       3
 
+typedef Bit32u apic_dest_t; /* same definition in ioapic.h */
+
 class BOCHSAPI bx_local_apic_c : public logfunctions
 {
   bx_phy_address base_addr;
   unsigned mode;
+  bx_bool xapic;
   Bit32u apic_id;               //  4 bit in legacy mode, 8 bit in XAPIC mode
+                                // 32 bit in X2APIC mode
+  Bit32u apic_version_id;
 
   bx_bool software_enabled;
   Bit8u  spurious_vector;
@@ -72,6 +71,7 @@ class BOCHSAPI bx_local_apic_c : public logfunctions
 #define APIC_ERR_ILLEGAL_ADDR    0x80
 #define APIC_ERR_RX_ILLEGAL_VEC  0x40
 #define APIC_ERR_TX_ILLEGAL_VEC  0x20
+#define X2APIC_ERR_REDIRECTIBLE_IPI 0x08
 #define APIC_ERR_RX_ACCEPT_ERR   0x08
 #define APIC_ERR_TX_ACCEPT_ERR   0x04
 #define APIC_ERR_RX_CHECKSUM     0x02
@@ -123,12 +123,16 @@ public:
   bx_phy_address get_base(void) const { return base_addr; }
   void set_base(bx_phy_address newbase);
   Bit32u get_id() const { return apic_id; }
+  bx_bool is_xapic() const { return xapic; }
   bx_bool is_selected(bx_phy_address addr);
   void read(bx_phy_address addr, void *data, unsigned len);
   void write(bx_phy_address addr, void *data, unsigned len);
   void write_aligned(bx_phy_address addr, Bit32u data);
   Bit32u read_aligned(bx_phy_address address);
-  void startup_msg(Bit8u vector);
+#if BX_SUPPORT_X2APIC
+  bx_bool read_x2apic(unsigned index, Bit64u *msr);
+  bx_bool write_x2apic(unsigned index, Bit64u msr);
+#endif
   // on local APIC, trigger means raise the CPU's INTR line. For now
   // I also have to raise pc_system.INTR but that should be replaced
   // with the cpu-specific INTR signals.
@@ -137,11 +141,11 @@ public:
   Bit8u acknowledge_int(void);  // only the local CPU should call this
   int highest_priority_int(Bit8u *array);
   void receive_EOI(Bit32u value);
-  void send_ipi(void);
+  void send_ipi(apic_dest_t dest, Bit32u lo_cmd);
   void write_spurious_interrupt_register(Bit32u value);
   void service_local_apic(void);
   void print_status(void);
-  bx_bool match_logical_addr (Bit8u address);
+  bx_bool match_logical_addr(apic_dest_t address);
   bx_bool deliver(Bit8u vector, Bit8u delivery_mode, Bit8u trig_mode);
   Bit8u get_tpr(void) { return task_priority; }
   void  set_tpr(Bit8u tpr);
@@ -152,11 +156,12 @@ public:
   void periodic(void);
   void set_divide_configuration(Bit32u value);
   void set_initial_timer_count(Bit32u value);
+  void startup_msg(Bit8u vector);
   void register_state(bx_param_c *parent);
 };
 
-int apic_bus_deliver_lowest_priority(Bit8u vector, Bit8u dest, bx_bool trig_mode, bx_bool broadcast);
-int apic_bus_deliver_interrupt(Bit8u vector, Bit8u dest, Bit8u delivery_mode, bx_bool logical_dest, bx_bool level, bx_bool trig_mode);
+int apic_bus_deliver_lowest_priority(Bit8u vector, apic_dest_t dest, bx_bool trig_mode, bx_bool broadcast);
+int apic_bus_deliver_interrupt(Bit8u vector, apic_dest_t dest, Bit8u delivery_mode, bx_bool logical_dest, bx_bool level, bx_bool trig_mode);
 int apic_bus_broadcast_interrupt(Bit8u vector, Bit8u delivery_mode, bx_bool trig_mode, int exclude_cpu);
 
 #endif // if BX_SUPPORT_APIC
