@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: arith16.cc,v 1.74 2010/02/26 11:44:50 sshwarts Exp $
+// $Id: arith16.cc,v 1.76 2011/01/25 20:59:26 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2009  The Bochs Project
@@ -26,13 +26,13 @@
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::INC_RX(bxInstruction_c *i)
 {
-  Bit16u rx = ++BX_READ_16BIT_REG(i->opcodeReg());
+  Bit16u rx = ++BX_READ_16BIT_REG(i->rm());
   SET_FLAGS_OSZAPC_INC_16(rx);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::DEC_RX(bxInstruction_c *i)
 {
-  Bit16u rx = --BX_READ_16BIT_REG(i->opcodeReg());
+  Bit16u rx = --BX_READ_16BIT_REG(i->rm());
   SET_FLAGS_OSZAPC_DEC_16(rx);
 }
 
@@ -416,11 +416,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_EwIwR(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::NEG_EwM(bxInstruction_c *i)
 {
-  Bit16u op1_16;
-
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
+  Bit16u op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op1_16 = - (Bit16s)(op1_16);
   write_RMW_virtual_word(op1_16);
 
@@ -438,11 +436,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::NEG_EwR(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::INC_EwM(bxInstruction_c *i)
 {
-  Bit16u op1_16;
-
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
+  Bit16u op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op1_16++;
   write_RMW_virtual_word(op1_16);
 
@@ -451,11 +447,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INC_EwM(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::DEC_EwM(bxInstruction_c *i)
 {
-  Bit16u op1_16;
-
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
+  Bit16u op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op1_16--;
   write_RMW_virtual_word(op1_16);
 
