@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: bit16.cc,v 1.19 2010/02/25 22:04:30 sshwarts Exp $
+// $Id: bit16.cc,v 1.20 2010/10/18 22:19:45 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2009  The Bochs Project
@@ -82,15 +82,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EwGwM(bxInstruction_c *i)
   index = op2_16 & 0x0f;
   displacement32 = ((Bit16s) (op2_16&0xfff0)) / 16;
   op1_addr = eaddr + 2 * displacement32;
-  if (! i->as32L())
-    op1_addr = (Bit16u) op1_addr;
-#if BX_SUPPORT_X86_64
-  else if (! i->as64L())
-    op1_addr = (Bit32u) op1_addr;
-#endif
 
   /* pointer, segment address pair */
-  op1_16 = read_virtual_word(i->seg(), op1_addr);
+  op1_16 = read_virtual_word(i->seg(), op1_addr & i->asize_mask());
 
   set_CF((op1_16 >> index) & 0x01);
 }
@@ -118,15 +112,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EwGwM(bxInstruction_c *i)
   index = op2_16 & 0x0f;
   displacement32 = ((Bit16s) (op2_16 & 0xfff0)) / 16;
   op1_addr = eaddr + 2 * displacement32;
-  if (! i->as32L())
-    op1_addr = (Bit16u) op1_addr;
-#if BX_SUPPORT_X86_64
-  else if (! i->as64L())
-    op1_addr = (Bit32u) op1_addr;
-#endif
 
   /* pointer, segment address pair */
-  op1_16 = read_RMW_virtual_word(i->seg(), op1_addr);
+  op1_16 = read_RMW_virtual_word(i->seg(), op1_addr & i->asize_mask());
   bit_i = (op1_16 >> index) & 0x01;
   op1_16 |= (((Bit16u) 1) << index);
   write_RMW_virtual_word(op1_16);
@@ -160,15 +148,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EwGwM(bxInstruction_c *i)
   index = op2_16 & 0x0f;
   displacement32 = ((Bit16s) (op2_16&0xfff0)) / 16;
   op1_addr = eaddr + 2 * displacement32;
-  if (! i->as32L())
-    op1_addr = (Bit16u) op1_addr;
-#if BX_SUPPORT_X86_64
-  else if (! i->as64L())
-    op1_addr = (Bit32u) op1_addr;
-#endif
 
   /* pointer, segment address pair */
-  op1_16 = read_RMW_virtual_word(i->seg(), op1_addr);
+  op1_16 = read_RMW_virtual_word(i->seg(), op1_addr & i->asize_mask());
   bx_bool temp_cf = (op1_16 >> index) & 0x01;
   op1_16 &= ~(((Bit16u) 1) << index);
 
@@ -204,14 +186,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EwGwM(bxInstruction_c *i)
   index_16 = op2_16 & 0x0f;
   displacement16 = ((Bit16s) (op2_16 & 0xfff0)) / 16;
   op1_addr = eaddr + 2 * displacement16;
-  if (! i->as32L())
-    op1_addr = (Bit16u) op1_addr;
-#if BX_SUPPORT_X86_64
-  else if (! i->as64L())
-    op1_addr = (Bit32u) op1_addr;
-#endif
 
-  op1_16 = read_RMW_virtual_word(i->seg(), op1_addr);
+  op1_16 = read_RMW_virtual_word(i->seg(), op1_addr & i->asize_mask());
   bx_bool temp_CF = (op1_16 >> index_16) & 0x01;
   op1_16 ^= (((Bit16u) 1) << index_16);  /* toggle bit */
   write_RMW_virtual_word(op1_16);
