@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: ctrl_xfer64.cc,v 1.84 2010/03/14 15:51:26 sshwarts Exp $
+// $Id: ctrl_xfer64.cc,v 1.85 2010/10/18 22:19:45 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2009  The Bochs Project
@@ -155,7 +155,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CALL64_Ep(bxInstruction_c *i)
 
   /* pointer, segment address pair */
   Bit64u op1_64 = read_virtual_qword_64(i->seg(), eaddr);
-  Bit16u cs_raw = read_virtual_word_64(i->seg(), eaddr+8);
+  Bit16u cs_raw = read_virtual_word_64(i->seg(), (eaddr+8) & i->asize_mask());
 
   BX_ASSERT(protected_mode());
 
@@ -410,7 +410,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::JMP64_Ep(bxInstruction_c *i)
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   Bit64u op1_64 = read_virtual_qword_64(i->seg(), eaddr);
-  Bit16u cs_raw = read_virtual_word_64(i->seg(), eaddr+8);
+  Bit16u cs_raw = read_virtual_word_64(i->seg(), (eaddr+8) & i->asize_mask());
 
   BX_ASSERT(protected_mode());
 
